@@ -1,6 +1,6 @@
 /* Structure type implementation */
 /*
-Copyright (C) 2004-2014 John E. Davis
+Copyright (C) 2004-2017,2018 John E. Davis
 
 This file is part of the S-Lang Library.
 
@@ -57,10 +57,26 @@ static void free_struct (_pSLang_Struct_Type *s)
 
    if (s->destroy_method != NULL)
      {
-	if ((0 == SLang_start_arg_list ())
-	    && (0 == SLang_push_struct (s))
-	    && (0 == SLang_end_arg_list ()))
-	  (void) SLexecute_function (s->destroy_method);
+	int err, status;
+
+	if ((0 != (err = _pSLang_Error))
+	    && (-1 == _pSLang_push_error_context ()))
+	  {
+	     SLang_free_function (s->destroy_method);
+	     free_fields (s->fields, s->nfields);
+	     SLfree ((char *) s);
+	     return;
+	  }
+
+	status = 0;
+	if ((-1 == SLang_start_arg_list ())
+	    || (-1 == SLang_push_struct (s))
+	    || (-1 == SLang_end_arg_list ())
+	    || (-1 == SLexecute_function (s->destroy_method)))
+	  status = -1;
+
+	if (err)
+	  _pSLang_pop_error_context (status != 0);
 
 	if (s->num_refs > 1)
 	  {
@@ -70,7 +86,6 @@ static void free_struct (_pSLang_Struct_Type *s)
 
 	SLang_free_function (s->destroy_method);
      }
-
    free_fields (s->fields, s->nfields);
    SLfree ((char *) s);
 }
@@ -136,7 +151,6 @@ int SLang_push_struct (_pSLang_Struct_Type *s)
 int SLang_pop_struct (_pSLang_Struct_Type **sp)
 {
    SLang_Object_Type obj;
-   SLang_Class_Type *cl;
    SLtype type;
 
    if (0 != SLang_pop (&obj))
@@ -145,6 +159,7 @@ int SLang_pop_struct (_pSLang_Struct_Type **sp)
    type = obj.o_data_type;
    if (type != SLANG_STRUCT_TYPE)
      {
+	SLang_Class_Type *cl;
 	cl = _pSLclass_get_class (type);
 	if (cl->is_struct == 0)
 	  {
@@ -200,13 +215,15 @@ static _pSLstruct_Field_Type *find_field_strcmp (_pSLang_Struct_Type *s, SLCONST
 {
    _pSLstruct_Field_Type *f, *fmax;
 
+   if (NULL != (f = find_field_in_fields (s->fields, s->nfields, name)))
+     return f;
+
    f = s->fields;
    fmax = f + s->nfields;
 
    while (f < fmax)
      {
-	if ((name == f->name)
-	    || (0 == strcmp (name, f->name)))
+	if (0 == strcmp (name, f->name))
 	  return f;
 
 	f++;
@@ -804,14 +821,23 @@ static int push_struct_of_type (SLtype type, _pSLang_Struct_Type *s);
 #define NUM_BINARY_OPS	(SLANG_BINARY_OP_MAX-SLANG_BINARY_OP_MIN+1)
 #define NUM_UNARY_OPS	(SLANG_UNARY_OP_MAX-SLANG_UNARY_OP_MIN+1)
 
+typedef struct Binary_Op_List_Type_
+{
+   SLtype type;
+   SLang_Class_Type *result_cl;
+   SLang_Name_Type *binary_func;
+   struct Binary_Op_List_Type_ *next;
+}
+Binary_Op_List_Type;
+
 typedef struct
 {
-   SLang_Class_Type *result_any_this_cl;
-   SLang_Class_Type *result_this_any_cl;
-   SLang_Class_Type *result_this_this_cl;
-   SLang_Name_Type *any_binary_this;
-   SLang_Name_Type *this_binary_void;
-   SLang_Name_Type *this_binary_this;
+   SLang_Name_Type *this_op_any;
+   SLang_Class_Type *result_this_op_any_cl;
+   SLang_Name_Type *any_op_this;
+   SLang_Class_Type *result_any_op_this_cl;
+   Binary_Op_List_Type *that_op_this_list;
+   Binary_Op_List_Type *this_op_that_list;
 }
 Binary_Op_Info_Type;
 
@@ -982,48 +1008,50 @@ static int struct_unary (int op, SLtype a_type, VOID_STAR ap, SLuindex_Type na,
    return -1;
 }
 
-static int this_binary_any_result (int op, SLtype a, SLtype b, SLtype *result)
+static Binary_Op_List_Type *find_type_in_binary_list (Binary_Op_List_Type *list, SLtype type)
+{
+   while ((list != NULL) && (list->type != type))
+     list = list->next;
+
+   return list;
+}
+
+static int this_op_any_result (int op, SLtype a, SLtype b, SLtype *result)
 {
    Binary_Op_Info_Type *bi;
+   Binary_Op_List_Type *item;
    SLang_Class_Type *cl;
 
-   (void) b;
    if (NULL == (bi = find_binary_info (op, a)))
      return 0;
 
-   if (NULL == (cl = bi->result_this_any_cl))
+   if (NULL != (item = find_type_in_binary_list (bi->this_op_that_list, b)))
+     cl = item->result_cl;
+   else
+     cl = bi->result_this_op_any_cl;
+
+   if (cl == NULL)
      return 0;
 
    *result = cl->cl_data_type;
    return 1;
 }
 
-static int this_binary_this_result (int op, SLtype a, SLtype b, SLtype *result)
+static int any_op_this_result (int op, SLtype a, SLtype b, SLtype *result)
 {
    Binary_Op_Info_Type *bi;
+   Binary_Op_List_Type *item;
    SLang_Class_Type *cl;
 
-   (void) b;
-   if (NULL == (bi = find_binary_info (op, a)))
-     return 0;
-
-   if (NULL == (cl = bi->result_this_this_cl))
-     return 0;
-
-   *result = cl->cl_data_type;
-   return 1;
-}
-
-static int any_binary_this_result (int op, SLtype a, SLtype b, SLtype *result)
-{
-   Binary_Op_Info_Type *bi;
-   SLang_Class_Type *cl;
-
-   (void) a;
    if (NULL == (bi = find_binary_info (op, b)))
      return 0;
 
-   if (NULL == (cl = bi->result_any_this_cl))
+   if (NULL != (item = find_type_in_binary_list (bi->that_op_this_list, a)))
+     cl = item->result_cl;
+   else
+     cl = bi->result_any_op_this_cl;
+
+   if (cl == NULL)
      return 0;
 
    *result = cl->cl_data_type;
@@ -1084,12 +1112,15 @@ static int do_struct_binary (SLang_Name_Type *function,
    return -1;
 }
 
-static int this_binary_any (int op,
-			     SLtype a, VOID_STAR ap, SLuindex_Type na,
-			     SLtype b, VOID_STAR bp, SLuindex_Type nb,
-			     VOID_STAR cp)
+static int this_op_any (int op,
+			SLtype a, VOID_STAR ap, SLuindex_Type na,
+			SLtype b, VOID_STAR bp, SLuindex_Type nb,
+			VOID_STAR cp)
 {
    Binary_Op_Info_Type *bi;
+   Binary_Op_List_Type *item;
+   SLang_Name_Type *nt;
+   SLang_Class_Type *cl;
 
    if (NULL == (bi = find_binary_info (op, a)))
      {
@@ -1097,18 +1128,32 @@ static int this_binary_any (int op,
 	return -1;
      }
 
-   return do_struct_binary (bi->this_binary_void,
+   if (NULL != (item = find_type_in_binary_list (bi->this_op_that_list, b)))
+     {
+	nt = item->binary_func;
+	cl = item->result_cl;
+     }
+   else
+     {
+	nt = bi->this_op_any;
+	cl = bi->result_this_op_any_cl;
+     }
+
+   return do_struct_binary (nt,
 			    _pSLclass_get_class (a), ap, na,
 			    _pSLclass_get_class (b), bp, nb,
-			    bi->result_this_any_cl, cp);
+			    cl, cp);
 }
 
-static int any_binary_this (int op,
-			     SLtype a, VOID_STAR ap, SLuindex_Type na,
-			     SLtype b, VOID_STAR bp, SLuindex_Type nb,
-			     VOID_STAR cp)
+static int any_op_this (int op,
+			SLtype a, VOID_STAR ap, SLuindex_Type na,
+			SLtype b, VOID_STAR bp, SLuindex_Type nb,
+			VOID_STAR cp)
 {
    Binary_Op_Info_Type *bi;
+   Binary_Op_List_Type *item;
+   SLang_Name_Type *nt;
+   SLang_Class_Type *cl;
 
    if (NULL == (bi = find_binary_info (op, b)))
      {
@@ -1116,29 +1161,21 @@ static int any_binary_this (int op,
 	return -1;
      }
 
-   return do_struct_binary (bi->any_binary_this,
-			    _pSLclass_get_class (a), ap, na,
-			    _pSLclass_get_class (b), bp, nb,
-			    bi->result_any_this_cl, cp);
-}
-
-static int this_binary_this (int op,
-			     SLtype a, VOID_STAR ap, SLuindex_Type na,
-			     SLtype b, VOID_STAR bp, SLuindex_Type nb,
-			     VOID_STAR cp)
-{
-   Binary_Op_Info_Type *bi;
-
-   if (NULL == (bi = find_binary_info (op, a)))
+   if (NULL != (item = find_type_in_binary_list (bi->that_op_this_list, a)))
      {
-	_pSLang_verror (SL_INTERNAL_ERROR, "binary-op not supported");
-	return -1;
+	nt = item->binary_func;
+	cl = item->result_cl;
+     }
+   else
+     {
+	nt = bi->any_op_this;
+	cl = bi->result_any_op_this_cl;
      }
 
-   return do_struct_binary (bi->this_binary_this,
+   return do_struct_binary (nt,
 			    _pSLclass_get_class (a), ap, na,
 			    _pSLclass_get_class (b), bp, nb,
-			    bi->result_this_this_cl, cp);
+			    cl, cp);
 }
 
 static int register_unary_ops (Struct_Info_Type *si, SLtype t)
@@ -1159,11 +1196,10 @@ static int register_binary_ops (Struct_Info_Type *si, SLtype t)
      return 0;
 
    if ((-1 == SLclass_add_binary_op (t, SLANG_VOID_TYPE,
-				     this_binary_any, this_binary_any_result))
+				     this_op_any, this_op_any_result))
        || (-1 == SLclass_add_binary_op (SLANG_VOID_TYPE, t,
-					any_binary_this, any_binary_this_result))
-       || (-1 == SLclass_add_binary_op (t, t,
-					this_binary_this, this_binary_this_result)))
+					any_op_this, any_op_this_result))
+       || (-1 == SLclass_add_binary_op (t, t, this_op_any, this_op_any_result)))
      return -1;
 
    si->binary_registered = 1;
@@ -1234,49 +1270,79 @@ static Binary_Op_Info_Type *find_binary_info (int op, SLtype t)
   return si->bi + op;
 }
 
+static int add_binary_op_to_list (Binary_Op_List_Type **listp,
+				  SLang_Class_Type *cl,
+				  SLang_Name_Type *nt, SLtype type)
+{
+   Binary_Op_List_Type *item, *list = *listp;
+
+   if (NULL == (item = find_type_in_binary_list (list, type)))
+     {
+	item = (Binary_Op_List_Type *)SLmalloc (sizeof(Binary_Op_List_Type));
+	if (item == NULL)
+	  return -1;
+	memset (item, 0, sizeof(Binary_Op_List_Type));
+	item->type = type;
+
+	if (list == NULL)
+	  *listp = item;
+	else
+	  {
+	     while (list->next != NULL) list = list->next;
+	     list->next = item;
+	  }
+	/* drop */
+     }
+   SLang_free_function (item->binary_func);
+   item->binary_func = nt;
+   item->result_cl = cl;
+   return 0;
+}
+
 static int add_binary_op (char *op,
 			  SLtype result_type, SLang_Name_Type *nt,
 			  SLtype a_type, SLtype b_type)
 {
    Binary_Op_Info_Type *bi;
-   int opcode;
    SLang_Class_Type *cl;
+   int opcode;
 
    if (-1 == (opcode = _pSLclass_get_binary_opcode (op)))
      return -1;
 
-   if (a_type == SLANG_ANY_TYPE)
-     bi = find_binary_info (opcode, b_type);
-   else
-     bi = find_binary_info (opcode, a_type);
-
-   if (bi == NULL)
-     return -1;
-
    cl = _pSLclass_get_class (result_type);
 
-   if (a_type != SLANG_ANY_TYPE)
+   if ((a_type == SLANG_ANY_TYPE)
+       || (NULL == find_struct_info (a_type, 0)))
      {
-	if (b_type == SLANG_ANY_TYPE)
+	/* something op this form */
+	bi = find_binary_info (opcode, b_type);
+	if (bi == NULL)
+	  return -1;
+	if (a_type == SLANG_ANY_TYPE)
 	  {
-	     if (bi->this_binary_void != NULL)
-	       SLang_free_function (bi->this_binary_void);
-	     bi->this_binary_void = nt;
-	     bi->result_this_any_cl = cl;
+	     SLang_free_function (bi->any_op_this);   /* NULL ok */
+	     bi->any_op_this = nt;
+	     bi->result_any_op_this_cl = cl;
 	     return 0;
 	  }
-	if (bi->this_binary_this != NULL)
-	  SLang_free_function (bi->this_binary_this);
-	bi->this_binary_this = nt;
-	bi->result_this_this_cl = cl;
+
+	return add_binary_op_to_list (&bi->that_op_this_list, cl, nt, a_type);
+     }
+
+   /* Otherwise this op something form */
+   bi = find_binary_info (opcode, a_type);
+   if (bi == NULL)
+     return -1;
+   if (b_type == SLANG_ANY_TYPE)
+     {
+	SLang_free_function (bi->this_op_any);   /* NULL ok */
+	bi->this_op_any = nt;
+	bi->result_this_op_any_cl = cl;
 	return 0;
      }
 
-   if (bi->any_binary_this != NULL)
-     SLang_free_function (bi->any_binary_this);
-   bi->any_binary_this = nt;
-   bi->result_any_this_cl = cl;
-   return 0;
+   return add_binary_op_to_list (&bi->this_op_that_list, cl, nt, b_type);
 }
 
 static int add_unary_op (char *op,
@@ -1795,7 +1861,6 @@ int _pSLstruct_define_typedef (void)
    SLang_free_slstring (type_name);
 
    cl->cl_struct_def = s1;
-   cl->cl_init_array_object = struct_init_array_object;
    cl->cl_datatype_deref = typedefed_struct_datatype_deref;
    cl->cl_destroy = struct_destroy;
    cl->cl_push = struct_push;
@@ -1804,6 +1869,7 @@ int _pSLstruct_define_typedef (void)
    cl->cl_foreach_close = struct_foreach_close;
    cl->cl_foreach = struct_foreach;
 
+   (void) SLclass_set_aelem_init_function (cl, struct_init_array_object);
    (void) SLclass_set_string_function (cl, string_method);
    (void) SLclass_set_eqs_function (cl, struct_eqs_method);
    (void) SLclass_set_acopy_function (cl, struct_acopy);
@@ -1916,15 +1982,57 @@ static void get_struct_field_names (_pSLang_Struct_Type *s)
    SLang_push_array (a, 1);
 }
 
-static int push_struct_fields (_pSLang_Struct_Type *s)
+static void push_struct_fields_intrin (void)
 {
+   _pSLang_Struct_Type *s;
    _pSLstruct_Field_Type *f, *fmax;
+   SLang_Array_Type *at = NULL;
    int num;
 
+   switch (SLang_Num_Function_Args)
+     {
+      default:
+	SLang_verror (SL_Usage_Error, "Usage: Incorrect number of arguments passed, expecting one or two arguments");
+	return;
+      case 2:
+	if (-1 == SLang_pop_array_of_type (&at, SLANG_STRING_TYPE))
+	  return;
+	/* drop */
+      case 1:
+	if (-1 == SLang_pop_struct (&s))
+	  {
+	     SLang_free_array (at);    /* NULL ok */
+	     return;
+	  }
+	break;
+     }
+
+   if (at != NULL)
+     {
+	char **namep, **namep_max;
+	int ret = 0;
+
+	namep = (char **)at->data;
+	namep_max = namep + at->num_elements;
+	while ((ret == 0) && (namep < namep_max))
+	  {
+	     /* Use strcmp method since there is no guarantee that the array is one of slstrings */
+	     if (NULL == (f = find_field_strcmp (s, *namep)))
+	       ret = SLang_push_null ();
+	     else
+	       ret = _pSLpush_slang_obj (&f->obj);
+
+	     namep++;
+	  }
+	SLang_free_array (at);
+	free_struct (s);
+	return;
+     }
+
    f = s->fields;
    fmax = f + s->nfields;
-
    num = 0;
+
    while (fmax > f)
      {
 	fmax--;
@@ -1933,8 +2041,8 @@ static int push_struct_fields (_pSLang_Struct_Type *s)
 
 	num++;
      }
-
-   return num;
+   SLang_free_struct (s);
+   (void) SLang_push_int (num);
 }
 
 /* Syntax: set_struct_field (s, name, value); */
@@ -2068,7 +2176,7 @@ static SLang_Intrin_Fun_Type Struct_Table [] =
 {
    MAKE_INTRINSIC_1("get_struct_field_names", get_struct_field_names, SLANG_VOID_TYPE, SLANG_STRUCT_TYPE),
    MAKE_INTRINSIC_1("get_struct_field", get_struct_field, SLANG_VOID_TYPE, SLANG_STRING_TYPE),
-   MAKE_INTRINSIC_1("_push_struct_field_values", push_struct_fields, SLANG_INT_TYPE, SLANG_STRUCT_TYPE),
+   MAKE_INTRINSIC_0("_push_struct_field_values", push_struct_fields_intrin, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("set_struct_field", struct_set_field, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("set_struct_fields", set_struct_fields, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("is_struct_type", is_struct_type, SLANG_INT_TYPE),
@@ -2412,6 +2520,7 @@ int _pSLstruct_push_field_ref (SLFUTURE_CONST char *name)
      {
 	SLang_free_struct (s);
 	SLang_free_slstring ((char *) name);
+	return -1;
      }
    frt = (Struct_Field_Ref_Type *) ref->data;
    frt->s = s;
